#!/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.6

$LastChangedRevision: 5843 $
$LastChangedDate:: 2022-05-03 07:58:28 +0200#$

This file is part of SyMSpace.

Copyright (c) 2017, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""
import os
import re
import shutil
import json
import glob
import numpy as np

from symspace.wrapper.symwrapper import SymProject

import componentspace.osutils.fileutils as fu
from componentspace.material_utils.material_functions import GeneralMaterialFunctions
from componentspace.material_utils.batch_material import MaterialBatchComponent
from componentspace.material_utils.ferromagnetic_material import MaterialFerromagnetic


class MaterialCogentPower(MaterialBatchComponent):
    """

    """

    def _createSingleCompfileFromTemplate(self, comp_file: str, material: dict):
        path_material = os.path.dirname(comp_file)
        component_name = material['component_name']

        '''Copy rst'''
        fu.copyFileAndReplace(os.path.join(self.path, 'index.rst'),
                              os.path.join(path_material, 'index.rst'),
                              [('__ComponentTitle__',
                                component_name + '\n' + len(component_name) * '='),
                               # ('__Comment__', material['comment']),
                               # ('__MaterialName__', material['name']),
                               # ('__Manufacturer__', material['manufacturer']),
                               ('__Material__', material['component_name']),
                               ('__BHCurve__', (component_name + '_bhcurve_plot.png')),
                               ('__PDF__', material['pdf_file']),
                               # ('__EXCEL__', material['excel_file'])
                               ])

        fu.copyTree(os.path.join(self.path, 'rst'), os.path.join(path_material, 'rst'))

        '''Copy pdf'''
        pdf_file = os.path.join(self.path, 'resources', material['pdf_file'])
        if os.path.isfile(pdf_file):
            shutil.copy(pdf_file, os.path.join(path_material, 'rst', material['pdf_file']))

        # '''Copy excel file'''
        # excel_file = os.path.join(self.path, 'resources', material['excel_file'])
        # if os.path.isfile(excel_file):
        #     shutil.copy(excel_file, os.path.join(path_material, 'rst', material['excel_file']))

        '''Copy png file'''
        png_file = os.path.join(path_material, 'rst', (component_name + '_bhcurve_plot.png'))
        MaterialFerromagnetic.plot_bhcurve(material, png_file)

        '''Modify material file'''
        prj = SymProject.load(comp_file)
        # prj['Project.Material.Type'].setData('Ferromagnetic material')
        # prj['Project.Material.Name'].setData(component_name)

        GeneralMaterialFunctions.set_filter(prj, material)
        GeneralMaterialFunctions.set_info(prj, material)
        GeneralMaterialFunctions.set_general_properties(prj, material)
        GeneralMaterialFunctions.set_electromagnetic_properties(prj, material)
        GeneralMaterialFunctions.set_mechanic_properties(prj, material)
        GeneralMaterialFunctions.set_thermal_properties(prj, material)

        prj.save()

    def extractMaterialData(self):
        # Todo: this needs to be cleaned up or re-implemented,
        #  so the functionality is split in smaller methods/functions.
        #  Many code lines are duplicates in MaterialFerromagetic, MaterialVoest and MaterialCogentPower
        from componentspace.material_utils.losscoefficient_functions import MaterialFunctions
        from componentspace.material_utils.bhcurve_functions import ExtendBHCurve, BHCurve_correction

        tasks = dict(only_correct=False, only_extend=True,
                     correct_extend=False, extend_correct=False, extend_correct_part=False)

        file_model = os.path.join(self.path, self.name + '.mop')
        if os.path.isfile(file_model):
            self.sym_mdl = SymProject.load(file_model)
        else:
            raise Exception('Model {} file does not exist!'.format(file_model))

        lst_val = []

        folder = str(os.path.join(self.path, "resources"))

        # ### Test with single file ####
        # all_materials_dictionary = {}
        # pdf_file = r'M400-50a.pdf'
        #
        # full_pdf_path = os.path.join(folder, pdf_file)
        #
        # ## read variables from the pdf-file (grade, Rp02, Rm, rho, ...)
        # pdf_dictionary = read_pdf_file(full_pdf_path, export=True)
        # pdf_dictionary['pdf_file'] = pdf_file

        '''All Files'''
        path_addition_data = str(os.path.join(folder, "additional_materialdata.json"))

        with open(path_addition_data, "r") as json_file:
            data = json.load(json_file)

        all_materials_dictionary = {}

        os.chdir(folder)
        # print(os.getcwd())
        for i, filename in enumerate(glob.glob('*.pdf')):
            print('**** {} ****'.format(filename))
            var1 = MaterialCogentPower.read_pdf_file(os.path.join(folder, filename))
            print(var1)
            var1['pdf_file'] = filename

            var_name_pdf = filename
            print("pdf_varname", var_name_pdf)

            all_materials_dictionary[var_name_pdf.replace('.pdf', '')] = var1
            all_materials_dictionary[var_name_pdf.replace('.pdf', '')]['class'] = 'Ferromagnetic material'
            print("data[filename] = ", data[filename])
            all_materials_dictionary[var_name_pdf.replace('.pdf', '')].update(data[filename])

        for key in all_materials_dictionary.keys():
            material_dictionary = all_materials_dictionary[key]

            # print(material_dictionary.keys())
            # print("DIN", material_dictionary['DIN EN 10106'])
            # print("IEC", material_dictionary['IEC'])
            material_dictionary['manufacturer'] = 'TataSteel'

            '''
            Calculation of the loss coefficients kefe and khfe
            !!! We need ONE value which is reasonable for most of the frequencies !!!
            !!! Additionally a function with respect to the frequency is useful   !!!
            !!! in the future                                                     !!!
            '''
            material_mp = MaterialFunctions.calc_loss_coefficients_MP(material_dictionary)

            material_dictionary['khfe_F'] = material_mp['khfe_F']
            material_dictionary['kefe_F'] = material_mp['kefe_F']
            material_dictionary['khfe'] = {"variable": "khfe", "value": material_mp['khfe'], "unit": "W.kg-1"}
            material_dictionary['kefe'] = {"variable": "kefe", "value": material_mp['kefe'], "unit": "W.kg-1"}

            '''
            Different versions of bh-curve extension and correction. The following versions where implemented:

            H_extrapolated, B_extrapolated = ExtendBHCurve.extend_bhcurve_arctan(material_dictionary)
            H_extrapolated, B_extrapolated = ExtendBHCurve.interpolate_bhcurve_arctan(material_dictionary)
            H_extrapolated, B_extrapolated = ExtendBHCurve.interpolate_bhcurve_mur(material_dictionary)
            H_extrapolated, B_extrapolated = ExtendBHCurve.extend_bhcurve(material_dictionary, type[0], False)

            !!! BHCurve_correction.correct_bhcurve - which is currently used should !!!
            !!! be replaced by a function implementing the Fröhlich-Kennelly extension !!!

            H_extrapolated, B_extrapolated = ExtendBHCurve.extend_bhcurve_froehlich_kennelly\
                (material_dictionary, material_dictionary['H_original'][0], material_dictionary['B_original'][0], False)
            '''

            mu_0 = 4 * np.pi * 1e-7
            H = material_dictionary['Magnetic field strength']['value']
            B = material_dictionary['Magnetic flux density']['value'][0]

            if H[0] > 0.0:
                H.insert(0, 0.0)
                B.insert(0, 0.0)

            material_dictionary['H_original'] = [H]
            material_dictionary['B_original'] = [B]

            '''Start: only correct'''
            if tasks['only_correct']:
                H_corrected, B_corrected = BHCurve_correction.correct_bhcurve \
                    (material_dictionary, material_dictionary['H_original'][0], material_dictionary['B_original'][0])
                material_dictionary['B_corrected'] = [B_corrected]
                material_dictionary['H_corrected'] = [H_corrected]
            '''End: only correct'''

            '''Start: only extend'''
            if tasks['only_extend']:
                # type = ['arctan', 'exp', 'logistic']
                # H_extrapolated, B_extrapolated = ExtendBHCurve.extend_bhcurve\
                #     (material_dictionary, type[0], material_dictionary['H_original'][0],
                #      material_dictionary['B_original'][0])
                H_extrapolated, B_extrapolated = ExtendBHCurve.extend_bhcurve_froehlich_kennelly \
                    (material_dictionary, material_dictionary['H_original'][0], material_dictionary['B_original'][0],
                     False)
                material_dictionary['B_extrapolated'] = [B_extrapolated.tolist()]
                material_dictionary['H_extrapolated'] = [H_extrapolated.tolist()]
                material_dictionary['B_corrected'] = [B_extrapolated.tolist()]
                material_dictionary['H_corrected'] = [H_extrapolated.tolist()]
                # print('H_extrapolated: ', H_extrapolated)
                # print('B_extrapolated: ', B_extrapolated)
                material_dictionary['B'] = [B_extrapolated.tolist()]
                material_dictionary['H'] = [H_extrapolated.tolist()]
            '''End: only extend'''

            '''Start: first correct, then extend'''
            if tasks['correct_extend']:
                H_corrected, B_corrected = BHCurve_correction.correct_bhcurve(material_dictionary,
                                                                              material_dictionary['H_original'][0],
                                                                              material_dictionary['B_original'][0],
                                                                              False)
                material_dictionary['B_corrected'] = [B_corrected]
                material_dictionary['H_corrected'] = [H_corrected]

                # type = ['arctan', 'exp', 'logistic']
                # H_extrapolated, B_extrapolated = ExtendBHCurve.extend_bhcurve\
                #     (material_dictionary, type[0], material_dictionary['H_corrected'][0],
                #      material_dictionary['B_corrected'][0])
                H_extrapolated, B_extrapolated = ExtendBHCurve.extend_bhcurve_froehlich_kennelly \
                    (material_dictionary, material_dictionary['H_corrected'][0], material_dictionary['B_corrected'][0],
                     False)
                material_dictionary['B_extrapolated'] = [B_extrapolated]
                material_dictionary['H_extrapolated'] = [H_extrapolated]
                print('H_extrapolated: ', H_extrapolated)
                print('B_extrapolated: ', B_extrapolated)
                material_dictionary['B'] = [B_extrapolated]
                material_dictionary['H'] = [H_extrapolated]
            '''End: first correct, then extend'''

            '''Start: first extend, then correct'''
            if tasks['extend_correct']:
                # type = ['arctan', 'exp', 'logistic']
                # H_extrapolated, B_extrapolated = ExtendBHCurve.extend_bhcurve\
                #     (material_dictionary, type[0], material_dictionary['H_original'][0],
                #      material_dictionary['B_original'][0])
                H_extrapolated, B_extrapolated = ExtendBHCurve.extend_bhcurve_froehlich_kennelly \
                    (material_dictionary, material_dictionary['H_original'][0], material_dictionary['B_original'][0],
                     False)
                material_dictionary['B_extrapolated'] = [B_extrapolated]
                material_dictionary['H_extrapolated'] = [H_extrapolated]
                # print('H_extrapolated: ', H_extrapolated)
                # print('B_extrapolated: ', B_extrapolated)

                H_corrected, B_corrected = BHCurve_correction.correct_bhcurve \
                    (material_dictionary, material_dictionary['H_extrapolated'][0],
                     material_dictionary['B_extrapolated'][0], False)
                material_dictionary['B_corrected'] = [B_corrected]
                material_dictionary['H_corrected'] = [H_corrected]
                material_dictionary['B'] = [B_corrected]
                material_dictionary['H'] = [B_corrected]
            '''End: first extend, then correct'''

            '''Start: first extend, then correct part'''
            if tasks['extend_correct_part']:
                # type = ['arctan', 'exp', 'logistic']
                # H_extrapolated, B_extrapolated = ExtendBHCurve.extend_bhcurve(material_dictionary, type[0], material_dictionary['H_original'][0], material_dictionary['B_original'][0])
                H_extrapolated, B_extrapolated = ExtendBHCurve.extend_bhcurve_froehlich_kennelly \
                    (material_dictionary, material_dictionary['H_original'][0], material_dictionary['B_original'][0],
                     False)
                material_dictionary['B_extrapolated'] = [B_extrapolated]
                material_dictionary['H_extrapolated'] = [H_extrapolated]
                # print('H_extrapolated: ', H_extrapolated)
                # print('B_extrapolated: ', B_extrapolated)

                B_Test = []
                B_Test.append(material_dictionary['B_extrapolated'][0][0])
                B_Rest = []
                H_Test = []
                H_Test.append(material_dictionary['H_extrapolated'][0][0])
                H_Rest = []
                for i in range(len(material_dictionary['H_extrapolated'][0]) - 1):
                    if (material_dictionary['B_extrapolated'][0][i + 1] - material_dictionary['B_extrapolated'][0][i]) \
                            / (
                            material_dictionary['H_extrapolated'][0][i + 1] - material_dictionary['H_extrapolated'][0][
                        i]) \
                            / mu_0 > 1.5:
                        B_Test.append(material_dictionary['B_extrapolated'][0][i + 1])
                        H_Test.append(material_dictionary['H_extrapolated'][0][i + 1])
                    else:
                        B_Rest.append(material_dictionary['B_extrapolated'][0][i + 1])
                        H_Rest.append(material_dictionary['H_extrapolated'][0][i + 1])

                H_corrected, B_corrected = BHCurve_correction.correct_bhcurve(material_dictionary, H_Test, B_Test,
                                                                              False)
                material_dictionary['B_corrected'] = [B_corrected].append(B_Rest)
                material_dictionary['H_corrected'] = [H_corrected].append(H_Rest)
            '''End: first extend, then correct part'''

            lst_val.append(material_dictionary)

            # select = ('Grade', 'Frequency [Hz]', 'Magnetic polarization [mT]', 'Magnetic field strength [A/m]',
            #           'Specific total loss [W/kg]', 'Relative permeability ')
            #

            #     row = list(ws.get_squared_range(ws.min_column, k, ws.max_column, k))[0]
            #     if row[title.index('Grade')].value is None:
            #         continue
            #
            #     val = {}
            #     for name in select:
            #         cell = row[title.index(name)].value
            #         # print(name + ' :: ' + str(cell))
            #         if name == 'Grade':
            #             val[name] = cell
            #             val['FileName'] = cell.replace('/', '_').replace(' ', '')
            #         elif isinstance(cell, str) and '~' in cell:
            #             rng = cell.split('~')
            #             val[name] = (float(rng[0].strip()) + float(rng[1].strip())) / 2
            #         elif isinstance(cell, str) and '-' in cell:
            #             rng = cell.split('-')
            #             try:
            #                 val[name] = (float(rng[0].strip()) + float(rng[1].strip())) / 2
            #             except:
            #                 val[name] = None
            #         else:
            #             val[name] = cell
            #
            #     lst_val.append(val)

            # print(material_dictionary)
            json_folder = str(os.path.join(self.path, "src"))
            with open(str(os.path.join(json_folder, material_dictionary['component_name'] + ".json")), "w") as f:
                json.dump(material_dictionary, f, indent=4, sort_keys=False)

    # # Read Excel data
    # def read_excel_file(excel_file):
    #     book = xlrd.open_workbook(excel_file, encoding_override="utf-8")
    #     sheet = book.sheet_by_index(0)
    #
    #     material_dictionary = {}
    #
    #     if 'Grade' == sheet.cell(1, 0).value:
    #         grade = np.asarray([sheet.cell(i, 0).value for i in range(2, sheet.nrows)])
    #         name = grade[0]
    #         material_dictionary['name'] = name
    #         material_dictionary['FileName'] = name.replace('isovac', 'M').replace(' ', '')
    #
    #         print('name = ', name)
    #         tlam = re.sub(r'^.*?-', '', name)
    #         print('tlam 1 = ', tlam)
    #         tlam = re.sub(r'A.*$', '', tlam)
    #         print('tlam 2 = ', tlam)
    #         material_dictionary['Lamination thickness'] = {'variable': 'tlam', 'value': float(tlam)/100.0, 'unit': 'mm'}
    #         if abs(float(tlam) - 35) < 0.1:
    #             material_dictionary['Stacking factor'] = {'variable': 'ks', 'value': 0.95, 'unit': '1'}
    #         elif abs(float(tlam) - 50) < 0.1:
    #             material_dictionary['Stacking factor'] = {'variable': 'ks', 'value': 0.96, 'unit': '1'}
    #         elif abs(float(tlam) - 65) < 0.1:
    #             material_dictionary['Stacking factor'] = {'variable': 'ks', 'value': 0.97, 'unit': '1'}
    #         elif abs(float(tlam) - 100) < 0.1:
    #             material_dictionary['Stacking factor'] = {'variable': 'ks', 'value': 0.98, 'unit': '1'}
    #         else:
    #             print('Stacking factor defined!')
    #
    #     else:
    #         print('No Grade defined!')
    #
    #     print('Frequency [Hz] == ', sheet.cell(1, 1).value)
    #     frequency = np.array([])
    #     indices_frequency = []
    #     if 'Frequency [Hz]' == sheet.cell(1, 1).value:
    #         temp_list = [sheet.cell(i, 1).value for i in range(2, sheet.nrows)]
    #         # temp_list = list(filter(None, temp_list))
    #         # frequency = np.array([float(i) for i in temp_list])
    #         temp_frequency = []
    #         counter = -1
    #         for item in temp_list:
    #             counter += 1
    #             if item:  # check for empty string
    #                 temp_frequency.append(float(item))
    #                 indices_frequency.append(counter)
    #
    #         indices_frequency.append(sheet.nrows - 2)  # work around since there is no frequency value in the last row
    #         frequency = np.array(temp_frequency)
    #         material_dictionary['Frequency'] = {'variable': 'F', 'value': frequency.tolist(), 'unit': 'Hz'}
    #     else:
    #         print('No Frequency defined!')
    #
    #     print('Magnetic polarization [mT] == ', sheet.cell(1, 2).value)
    #     temp_polarization = []
    #     if 'Magnetic polarization [mT]' == sheet.cell(1, 2).value:
    #         for i in range(len(indices_frequency) - 1):
    #             temp_list = [sheet.cell(i, 2).value for i in
    #                          range(2 + indices_frequency[i], 2 + indices_frequency[i + 1])]
    #             temp_list = [value for value in temp_list if value != 0.0 and value != ""]
    #             temp_list.insert(0, 0.0)
    #             temp_polarization.append(temp_list)
    #
    #         polarization = np.array(temp_polarization)
    #         material_dictionary['Magnetic polarization'] = {'variable': 'J', 'value': polarization.tolist(), 'unit': 'mT'}
    #     else:
    #         print('No Magnetic polarization defined!')
    #
    #     print('Magnetic field strength [A/m] == ', sheet.cell(1, 3).value)
    #     temp_field_strength = []
    #     if 'Magnetic field strength [A/m]' == sheet.cell(1, 3).value:
    #         for i in range(len(indices_frequency) - 1):
    #             temp_list = [sheet.cell(i, 3).value for i in
    #                          range(2 + indices_frequency[i], 2 + indices_frequency[i + 1])]
    #             temp_list = [value for value in temp_list if value != 0.0 and value != ""]
    #             temp_list.insert(0, 0.0)
    #             temp_field_strength.append(temp_list)
    #
    #         field_strength = np.array(temp_field_strength)
    #         material_dictionary['Magnetic field strength'] = {'variable': 'H', 'value': field_strength.tolist(), 'unit': 'A/m'}
    #     else:
    #         print('No Magnetic field strength defined!')
    #
    #     print('Specific total loss [W/kg] == ', sheet.cell(1, 4).value)
    #     temp_loss = []
    #     if 'Specific total loss [W/kg]' == sheet.cell(1, 4).value:
    #         for i in range(len(indices_frequency) - 1):
    #             temp_list = [sheet.cell(i, 4).value for i in
    #                          range(2 + indices_frequency[i], 2 + indices_frequency[i + 1])]
    #             temp_list = [value for value in temp_list if value != 0.0 and value != ""]
    #             temp_list.insert(0, 0.0)
    #             temp_loss.append(temp_list)
    #
    #         specific_loss = np.array(temp_loss)
    #         material_dictionary['Specific total loss'] = {'variable': 'P', 'value': specific_loss.tolist(), 'unit': 'W/kg'}
    #     else:
    #         print('No Specific total loss defined!')
    #
    #     temp_permeability = []
    #     if 'Relative permeability \n[-]' == sheet.cell(1, 5).value:
    #         for i in range(len(indices_frequency) - 1):
    #             temp_list = [sheet.cell(i, 5).value for i in
    #                          range(2 + indices_frequency[i], 2 + indices_frequency[i + 1])]
    #             temp_list = [value for value in temp_list if value != 0.0 and value != ""]
    #             temp_list.insert(0, 0.0)
    #             temp_permeability.append(temp_list)
    #
    #         permeability = np.array(temp_permeability)
    #         material_dictionary['Relative permeability'] = {'variable': 'mur', 'value': permeability.tolist(), 'unit': '-'}
    #     else:
    #         print('No Relative permeability defined!')
    #
    #     # start = 0
    #     # end = 0
    #     # for k in range(len(grade)):
    #     #     if grade[k] == name:
    #     #         end = k
    #     #         material['frequency'].append(float(frequency[k]))
    #     #         print(polarization)
    #     #         if start != end:
    #     #             material['polarization'].append(np.array([float(polarization[j]) for j in range(start, end) if polarization[j] != '' and polarization[j] != '0.0']))
    #     #             material['field_strength'].append(np.array([float(field_strength[j]) for j in range(start, end) if polarization[j] != '' and polarization[j] != '0.0']))
    #     #             material['specific_loss'].append(np.array([float(specific_loss[j]) for j in range(start, end) if polarization[j] != '' and polarization[j] != '0.0']))
    #     #             material['permeability'].append(np.array([float(permeability[j]) for j in range(start, end) if polarization[j] != '' and polarization[j] != '0.0']))
    #     #
    #     #         start = end
    #     #
    #     #     elif grade[k] != '' and grade[k] != name:
    #     #         end = k
    #     #         material['polarization'].append(np.array([float(polarization[j]) for j in range(start, end) if polarization[j] != '' and polarization[j] != '0.0']))
    #     #         material['field_strength'].append(np.array([float(field_strength[j]) for j in range(start, end) if polarization[j] != '' and polarization[j] != '0.0']))
    #     #         material['specific_loss'].append(np.array([float(specific_loss[j]) for j in range(start, end) if polarization[j] != '' and polarization[j] != '0.0']))
    #     #         material['permeability'].append(np.array([float(permeability[j]) for j in range(start, end) if polarization[j] != '' and polarization[j] != '0.0']))
    #
    #     return material_dictionary

    @staticmethod
    def read_pdf_file(pdf_file, export=False):
        """
        Use package camelot to extract table data from Voestalpine Electrical Steel datasheet.

        Notes:
        ------
        Tuple used in keywords dict has format:
            (delta_column, delta_row_variable, delta_row_unit, delta_row_value)

        flavor='stream' necessary!

        Parameters
        ----------
        pdf_file : str
            Full path to pdf file
        export : bool
            If True, an .csv file with the raw data is generated for debugging purpose (name: exported-... .csv).

        Returns
        -------
        variables : list of dict
            Every list entry contains information about the variable
        """
        import camelot  # pip install camelot-py

        mu_0 = 4 * np.pi * 1e-7

        std = (0, 1, 3)  # (variable, unit, value)
        keywords = {
            'Typical data for': std,
            'Resistivity': std,
            'Yield strength': std,
            'Tensile strength': std,
            'Hardness HV5 (VPN)': std,
            'Young’s modulus (RD)': std,
            'Young’s modulus (TD)': std
            # 'Specific electrical resistance': (1, 1, 2, 3),
            # 'Thermal conductivity': std,
            # # '1.0 T': (0, 1, 3, 4),  # Specific total loss @ 1.0T
            # # '1.5 T': (-1, 1, 3, 4),  # Specific total loss @ 1.5T
            # # '2500 A/m': (0, 1, 3, 4)  # Magnetic polarization @2500 A/m
            # 'DIN EN 10106': std,
            # 'IEC': std,
            # 'JIS': std,
            # 'GOST': std,
            # 'ASTM': std,
            # 'AISI': std,
            # 'IS648': std,
            # 'GB / T2521.1': std
        }

        tables = camelot.read_pdf(pdf_file, flavor='stream', pages='1', edge_tol=1000)  # read page # 2

        export = False
        if export:
            dir_path = os.path.dirname(os.path.realpath(pdf_file))
            out_file_path = os.path.join(dir_path, 'exported.csv')
            tables.export(out_file_path, f='csv')

        df = tables[1].df

        new_data = []
        for i in range(len(df.values)):
            new_data.append([])

            current_line = df.values[i]
            # current_line = np.delete(df.values[i], np.where(df.values[i] == ''))

            for j in range(len(current_line)):
                split_line = current_line[j].replace(", RD", " (RD)").replace(", TD", " (TD)").replace(" SURA® ",
                                                                                                       ", SURA ")
                split_line = split_line.replace(" \n", ", ")
                split_line = split_line.replace("\n", ", ")
                split_line = split_line.split(", ")
                for k in range(len(split_line)):
                    new_data[i].append(split_line[k])
        #         print(df.values[i][j].split(", "))

        length = max(map(len, new_data))
        data_array = np.array([x + [None] * (length - len(x)) for x in new_data])

        material_dictionary = {}
        for key, value in keywords.items():
            # print('** Search for {} **'.format(key))
            res = np.where(data_array == key)
            listOfCoordinates = list(zip(res[0], res[1]))
            entries_found = len(listOfCoordinates)
            if entries_found == 0:
                print('Keyword {} not found'.format(key))
            elif entries_found >= 2:
                raise Exception('Found keyword {} {} times'.format(key, entries_found))
            else:  # found keyword exactly 1x
                cord = listOfCoordinates[0]
                yc = cord[0]  # + value[0]

                if key == "Typical data for":
                    # Magnetic flux density [T]
                    col_mp = np.where(data_array[yc + 1] == "T")
                    counter = yc + 3

                    temp_magflux = []
                    temp_magflux.append(0.0)
                    current_value = data_array[counter][col_mp][0].replace(",", ".")
                    while GeneralMaterialFunctions.is_float(current_value):
                        temp_magflux.append(float(current_value))
                        counter = counter + 1
                        current_value = data_array[counter][col_mp][0].replace(",", ".")

                    magflux = np.array(temp_magflux)
                    material_dictionary['Magnetic flux density'] = {'variable': 'B', 'value': magflux.tolist(),
                                                                    'unit': 'T'}

                    # Magnetic field strength [A/m]
                    col_fs = np.where(data_array[yc + 1] == "A/m")
                    counter = yc + 3

                    temp_field_strength = []
                    temp_field_strength.append(0.0)
                    current_value = data_array[counter][col_fs][0].replace(",", ".")
                    while GeneralMaterialFunctions.is_float(current_value) and len(temp_field_strength) < len(
                            temp_magflux):
                        temp_field_strength.append(float(current_value))
                        counter = counter + 1
                        current_value = data_array[counter][col_fs][0].replace(",", ".")

                    field_strength = np.array(temp_field_strength)
                    material_dictionary['Magnetic field strength'] = {'variable': 'H', 'value': field_strength.tolist(),
                                                                      'unit': 'A/m'}

                    temp_polarization = []
                    for i in range(len(material_dictionary['Magnetic flux density']['value'])):
                        temp_polarization.append(
                            1000.0 * (material_dictionary['Magnetic flux density']['value'][i] - mu_0 *
                                      material_dictionary['Magnetic field strength']['value'][i]))

                    polarization = np.array(temp_polarization)
                    material_dictionary['Magnetic polarization'] = {'variable': 'J', 'value': polarization.tolist(),
                                                                    'unit': 'mT'}

                    # Frequency [Hz]
                    # Specific total loss [W/kg]
                    frequency = np.array([])
                    temp_frequency = []

                    temp_loss = []

                    magflux = material_dictionary['Magnetic flux density']['value']
                    magflux_list = []

                    col_p = np.where(data_array[yc + 1] == "W/kg")
                    row = yc + 2

                    shift = 0
                    if data_array[row][0] != '':
                        shift = -1

                    for col in col_p[0]:
                        counter = yc + 3
                        current_value = data_array[row][col + shift].replace("at ", "").replace(" Hz", "")
                        temp_frequency.append(float(current_value))

                        current_value = data_array[counter][col + shift].replace(",", ".")
                        temp_list = []
                        temp_list.append(0.0)
                        while GeneralMaterialFunctions.is_float(current_value) and len(temp_list) < len(temp_magflux):
                            temp_list.append(float(current_value))
                            counter = counter + 1
                            current_value = data_array[counter][col + shift].replace(",", ".")

                        temp_loss.append(temp_list)
                        magflux_list.append(magflux[0:len(temp_list)])

                    specific_loss = np.array(temp_loss)
                    material_dictionary['Specific total loss'] = {'variable': 'P', 'value': specific_loss.tolist(),
                                                                  'unit': 'W/kg'}

                    frequency = np.array(np.unique(temp_frequency, axis=0))
                    material_dictionary['Frequency'] = {'variable': 'F', 'value': frequency.tolist(), 'unit': 'Hz'}

                    # update: 'Magnetic polarization'
                    material_dictionary['Magnetic flux density'] = {'variable': 'B', 'value': magflux_list, 'unit': 'T'}

                else:
                    description = key
                    variable = data_array[yc][cord[1] + value[0]]
                    unit = MaterialCogentPower.to_symspace_unit(data_array[yc][cord[1] + value[1]])
                    value = str(data_array[yc][cord[1] + value[2]]).replace(',', '.')
                    s = '{}: {} = {} {}'.format(description, variable, value.replace(' ', ''), unit)  #
                    # print(s)
                    res = {
                        'variable': variable,
                        'value': value,
                        'unit': unit
                    }
                    material_dictionary[description] = res

        # Stacking factor: rule of thumb from https://www.emetor.com/glossary/stacking-factor/
        # for tlam in mm:
        # 0.0425 * tlam + 0.6032 * arctan(108.16 * tlam)
        print('name = ', pdf_file)
        tlam = re.sub(r'^.*?-', '', pdf_file)
        print('tlam 1 = ', tlam)
        tlam = re.sub(r'[A-z].*$', '', tlam)
        print('tlam 2 = ', tlam)
        material_dictionary['Lamination thickness'] = {'variable': 'tlam', 'value': float(tlam) / 100.0, 'unit': 'mm'}
        if abs(float(tlam) - 20) < 0.1:
            material_dictionary['Stacking factor'] = {'variable': 'ks', 'value': 0.93, 'unit': '1'}
        elif abs(float(tlam) - 35) < 0.1:
            material_dictionary['Stacking factor'] = {'variable': 'ks', 'value': 0.95, 'unit': '1'}
        elif abs(float(tlam) - 50) < 0.1:
            material_dictionary['Stacking factor'] = {'variable': 'ks', 'value': 0.96, 'unit': '1'}
        elif abs(float(tlam) - 65) < 0.1:
            material_dictionary['Stacking factor'] = {'variable': 'ks', 'value': 0.97, 'unit': '1'}
        elif abs(float(tlam) - 100) < 0.1:
            material_dictionary['Stacking factor'] = {'variable': 'ks', 'value': 0.98, 'unit': '1'}
        else:
            print('Stacking factor defined!')

        return material_dictionary

    @staticmethod
    def to_symspace_unit(input_unit):
        temp_unit = input_unit.replace('[', '').replace(']', '')

        if temp_unit == "N/mm²":
            symspace_unit = "[N.mm-2]"
        elif temp_unit == "-":
            symspace_unit = "[1]"
        elif temp_unit == "g/cm3":
            symspace_unit = "[g.cm-3]"
        elif temp_unit == "\u03bc\u03a9cm":
            symspace_unit = "[uOhm.cm]"
        elif temp_unit == "W/mK":
            symspace_unit = "[W.m-1.K-1]"
        elif temp_unit == "%":
            symspace_unit = "[%]"

        # elif temp_unit == "Material No.":
        #     symspace_unit = "[1]"
        # elif temp_unit == "M250-35A 5":
        #     symspace_unit = "[1]"
        # elif temp_unit == "M270-35A 5":
        #     symspace_unit = "[1]"
        # elif temp_unit == "35A250":
        #     symspace_unit = "[1]"
        # elif temp_unit == "2413":
        #     symspace_unit = "[1]"
        # elif temp_unit == "35C250":
        #     symspace_unit = "[1]"
        else:
            print("Conversion of " + input_unit + " not implemented.")
            symspace_unit = input_unit

        # if symspace_unit is None:
        #    raise Exception("Conversion of " + input_unit + " not implemented.")

        return symspace_unit
