#!/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.6

$LastChangedRevision: 4715 $
$LastChangedDate:: 2021-09-01 15:44:32 +0200#$

This file is part of SyMSpace.

Copyright (c) 2018, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""
import os
import symspace.osutils.processutils as pu
from componentspace.osutils.paths import ComponentPaths


class ComponentSVNUtils:
    """This class provides component related (build, test, etc.) svn utilities

       The main purpose of implementing this class was
       to make the functions accessible outside of java/jep/CSyMSpace
    """
    @classmethod
    def writeRevisionFile(cls, component_path: str) -> tuple:
        try:
            revision_str, error = pu.getSVNRevision(component_path)
            revision = int(revision_str) if error == '' else 0
        except Exception as ex:
            revision = 0
            error = str(ex)

        revision_txt = ComponentPaths.getRevisonFile(component_path)

        with open(revision_txt, 'w') as fh:
            fh.write(str(revision))

        return revision, error

    @classmethod
    def readRevisionFile(cls, component_path: str) -> int:
        """
        Get the component revision from the revision file

        Parameters
        ----------
        component_path : str
            Path to the component

        Returns
        -------
        revision : int
        """
        revision_txt = ComponentPaths.getRevisonFile(component_path)

        if os.path.isfile(revision_txt):
            with open(revision_txt, 'r') as fh:
                revision = int(fh.read())
        else:
            revision = 0
        return revision
