# !/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.6

$LastChangedRevision: 4715 $
$LastChangedDate:: 2021-09-01 15:44:32 +0200#$

This file is part of SyMSpace.

Copyright (c) 2021, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""
import os
import lxml.etree


class JUnitReader:
    """Class to read junit test results, created from SyMProjectTests in the ComponentSpace"""

    def __init__(self, junit_xml: str):
        """
        Initialize JUnitReader from xml file

        Parameters
        ----------
        junit_xml : str
            Path to the junit file
        """
        self.__tree = lxml.etree.parse(junit_xml)

    def findSyMProjectTestErrors(self) -> tuple:
        """
        Parse JUnit report to find which tests failed and which succeeded

        Returns
        -------
        comp_ok : list
            List of succeeding tests
        comp_errors : list
            List of failed tests
        """
        from symspace.testing.symproject.case import SymProjectTestCase

        comp_ok = []
        comp_errors = []
        # src_errors = []
        for index, element in enumerate(self.__tree.iter('testcase')):
            if element.tag == 'testcase':

                test_id = element.get('name')
                try:
                    '''try to figure out, to which component the testcase belongs'''
                    '''get the path of the test'''
                    prj_file = SymProjectTestCase.fileFromId(test_id)

                    '''ComponentRootFolder/**/tests/test_*.mop'''
                    comp_dir = os.path.dirname(os.path.dirname(prj_file))
                    root_dir = comp_dir
                    for key in ['_FreeCAD', '_Creo', 'Creo_MNF']:
                        '''If the tests paths end with e.g. "_FreeCAD" 
                           the root component folder will be returned
                           Stator1/Stator1_FreeCAD  or Stator1/Stator1_Creo -> Stator1'''
                        if comp_dir.endswith(key):
                            root_dir = os.path.dirname(comp_dir)
                            break

                    if len(tuple(element.iter('error'))) > 0:
                        '''add to the list of failed tests'''
                        comp_errors.append(root_dir)
                    else:
                        '''add to the list of successful tests'''
                        comp_ok.append(root_dir)

                except Exception as ex:
                    print('Error in name={}\n{}\n{}'.format(test_id, 'Error Message:', ex))

        '''if a root_path is contained both lists, remove it from the ok-list
            e.g. for Geometry components that have the same root path
            Stator1_FreeCAD (OK) + Stator1_Creo (Error) -> Stator1 (Error)
        '''
        comp_ok_final = list(set(comp_ok).difference(set(comp_errors)))

        return comp_ok_final, comp_errors
