# !/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.6

$LastChangedRevision: 5322 $
$LastChangedDate:: 2021-12-06 15:31:58 +0100#$

This file is part of SyMSpace.

Copyright (c) 2021, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""
from symspace.console.script_utils import ScriptUtils
from symspace.wrapper.symwrapper import SymProject, SymContainer, SymField, SymFunction

from symspace.gui.qt.form_dialog import FormDialog


def main(project: SymProject):
    form_title = 'Select Testparameter Settings'

    form_elements = [
        ('Remove Unused', 'checkbox', True),
        ('Remove empty containers', 'checkbox', True),
        ('Formula to Field', 'checkbox', True),
        ('Function to Container', 'checkbox', True),
        ('Remove ComponentProperty', 'checkbox', True),
    ]
    form_result = FormDialog.run(form_title, form_elements)

    if form_result['accept']:
        remove_unused = form_result['Remove Unused']
        formula_to_field = form_result['Formula to Field']
        function_to_container = form_result['Function to Container']
        remove_component_property = form_result['Remove ComponentProperty']

        if project.getObject('Testparameter') is not None:
            for _ in range(10):
                '''run multiple times just to be sure'''
                params = project.getObject('Testparameter').getParameters('SymParameter')
                for param in params:
                    if isinstance(param, SymContainer):
                        if function_to_container and hasattr(param, 'convertToSymContainer'):
                            print('"{}": Convert to container'.format(param.getShortIdentifier()))
                            param = param.convertToSymContainer()
                        if param.getParamSize() == 0:
                            print('"{}": Removed param'.format(param.getShortIdentifier()))
                            param.removeParam()
                    if isinstance(param, (SymContainer, SymFunction)):
                        if remove_component_property and param.isComponent():
                            print('"{}": Remove ComponentProperty'.format(param.getShortIdentifier()))
                            param.removeComponentProperty()
                    if isinstance(param, SymField):
                        if formula_to_field and hasattr(param, 'toSymField'):
                            print('"{}": Convert to field'.format(param.getShortIdentifier()))
                            param = param.toSymField()
                        if remove_unused and not param.getUsed():
                            print('"{}": Removed param'.format(param.getShortIdentifier()))
                            param.removeParam()
    else:
        print('Cancelled by user')


if __name__ == '__main__':
    """
    This script inserts a test parameter for unittest

    This script needs to be executed in the SyMSpace Python Console.
    """

    _params = locals()
    _project = ScriptUtils.getProject(_params)
    if _project is not None:
        ScriptUtils.run(main, _project)
    else:
        ScriptUtils.showNoProjectWarning()
