#!/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.6

$LastChangedRevision: 5343 $
$LastChangedDate:: 2021-12-13 15:25:29 +0100#$

This file is part of SyMSpace.

Copyright (c) 2018, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""

if __name__ == '__main__':
    """
    This script lists all component project tests

    This script needs to be run in a SyMSpace console!

    Commandline Arguments
    ---------------------

    --start_dir : str
        The search for tests starts from this directory (Optional).
        Default is the ComponentSpace root directory.


    Examples
    --------

    >>> CSyMSpace.exe list_component_tests.py

    >>> CSyMSpace.exe list_component_tests.py --start_dir "C:\\ComponentSpace\\Simulation\\Motor"

    """
    from symspace.osutils.pathutils import Paths
    from symspace.testing.symproject.suite import SymProjectTestSuite
    from componentspace.testing.create_tests import ComponentTestBuilder

    start_dir, _, _ = SymProjectTestSuite.getCMDArguments(
        default_start_dir=Paths.getComponentSpaceRoot(), default_process_timeout=300)

    suite = SymProjectTestSuite()
    prj_files = suite.findTestsFilesByPattern(start_dir, ComponentTestBuilder.get_search_pattern())

    n_files = len(prj_files)

    print('Found {} below {}\n'.format(n_files, start_dir))

    for index, file in enumerate(prj_files):
        print('{}/{} {}'.format(index + 1, n_files, file))

